<?php

namespace App\Http\Controllers;

use App\Models\Brand;
use App\Models\Category;
use App\Models\Classe;
use App\Models\Order;
use App\Models\OrderDetail;
use App\Models\Product;
use App\Models\StockModel;
use Illuminate\Http\Request;

/**
 * Class ProductController
 * @package App\Http\Controllers
 */
class ProductController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $products = Product::query()->paginate();

        return view('product.index', compact('products'))
            ->with('i', (request()->input('page', 1) - 1) * $products->perPage());
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $product = new Product();

        $brands = Brand::query()->pluck('name', 'id');
        $categories = Category::query()->pluck('name', 'id');
        $classes = Classe::query()->pluck('name', 'id');

        return view('product.create', compact('product', 'brands', 'categories', 'classes'));
    }

   
    public function buscarProductosPorMarca($brand_id, Request $request)
{
    $search = $request->get('term');

    // Obtener productos de StockModel filtrados por brand_id y nombre
    $productosStock = StockModel::where('brand_id', $brand_id)
        ->where('nombre', 'LIKE', "%{$search}%")
        ->get();

    // Obtener productos de OrderDetail relacionados con productos que coincidan con el brand_id
    $productosOrderDetail = OrderDetail::whereHas('product', function ($query) use ($brand_id, $search) {
        $query->where('brand_id', $brand_id)
              ->where('name', 'LIKE', "%{$search}%")
              ->where('state', "DONE");
    })
    ->get()
    ->map(function ($orderDetail) {
        return [
            'id' => $orderDetail->id,
            'label' => ($orderDetail->product->name ?? 'Producto Desconocido') . ' (Orden - ' . $orderDetail->quantity-$orderDetail->quantity_prestado . ')',
            'value' => ($orderDetail->product->name ?? 'Producto Desconocido') . ' (Orden - ' . $orderDetail->quantity-$orderDetail->quantity_prestado . ')',
            'is_order_detail' => true,
            'order_id' => $orderDetail->order_id, // ID de la orden
            'quantity' => $orderDetail->quantity,
            'quantity_prestado' => $orderDetail->quantity_prestado,
        ];
    });

    // Transformar productos de StockModel
    $productosStock = $productosStock->map(function ($producto) {
        return [
            'id' => $producto->id,
            'label' => ($producto->nombre) . ' (' . $producto->cantidad . ')',
            'value' => $producto->nombre . ' (' . $producto->cantidad . ')',
            'is_order_detail' => false,
            'order_id' => null, // No hay orden asociada
            'cantidad' => $producto->cantidad,
        ];
    });

    // Combinar ambas colecciones
    $productos = $productosStock->concat($productosOrderDetail);

    return response()->json($productos);
}


    
   


    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        request()->validate(Product::$rules);

        $product = Product::create($request->all());

        if($request->ajax()){
            $order = Order::query()->find($request->order);
            $orderDetail = OrderDetail::query()->create([
                'order_id' => $request->order,
                'product_id' => $product->id,
            ]);

            return response()->json([
                'success' => 'Producto creado con éxito.',
                    'detail' => $orderDetail->id,
                'data' => [
                    'id' => $product->id,
                    'name' => $product->name,
                    'description' => $product->description,
                    'brand' => $product->brand->name,
                    'category' => $product->category->name,
                    'class' => $product->classe->name,
                ],
                    'html' => view('order.row', compact('product', 'orderDetail', 'order'))->render(),
                ]
            );
        }

        return redirect()->route('products.index')
            ->with('success', 'Producto creado con éxito.');
    }

    /**
     * Display the specified resource.
     *
     * @param  int $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $product = Product::find($id);

        return view('product.show', compact('product'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $product = Product::find($id);

        $brands = Brand::query()->pluck('name', 'id');
        $categories = Category::query()->pluck('name', 'id');
        $classes = Classe::query()->pluck('name', 'id');

        return view('product.edit', compact('product', 'brands', 'categories', 'classes'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request $request
     * @param  Product $product
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Product $product)
    {
        request()->validate(Product::$rules);

        $product->update($request->all());

        return redirect()->route('products.index')
            ->with('success', 'Producto actualizado con éxito');
    }

    /**
     * @param int $id
     * @return \Illuminate\Http\RedirectResponse
     * @throws \Exception
     */
    public function destroy($id)
    {
        $product = Product::find($id)->delete();

        return redirect()->route('products.index')
            ->with('success', 'Product deleted successfully');
    }

    public function search(Request $request)
    {
        $productQuery = Product::query()->select('products.*')
            ->join('brands', 'brands.id', '=', 'products.brand_id')
            ->orderBy('brands.name');

        if($request->has('name') && $request->name){
            $productQuery->where('name', 'like', '%'.$request->name.'%');
        }

        if($request->has('brand') && $request->brand){
            $productQuery->where('brand_id', $request->brand);
        }

        return view('product.search', ['products' => $productQuery->get(), 'i' => 0]);
    }

    public function addProduct(Request $request)
    {
        $order = Order::query()->find($request->order);
        $product = Product::query()->find($request->product);
        $orderDetail = OrderDetail::query()->create([
            'order_id' => $request->order,
            'product_id' => $product->id,
        ]);

        return response()->json([
                'success' => 'Producto creado con éxito.',
                'detail' => $orderDetail->id,
                'data' => [
                    'id' => $product->id,
                    'name' => $product->name,
                    'description' => $product->description,
                    'brand' => $product->brand->name,
                    'category' => $product->category->name,
                    'class' => $product->classe->name,
                ],
                'html' => view('order.row', compact('product', 'orderDetail', 'order'))->render(),
            ]
        );
    }
}
