<?php

namespace App\Http\Controllers;

use App\Jobs\SendDocumentExternalJob;
use App\Jobs\SendOrderToCustomerJob;
use App\Models\Action;
use App\Models\Brand;
use App\Models\Customer;
use App\Models\Order;
use App\Models\OrderDetail;
use App\Models\Refill;
use App\Models\Usage;
use App\Models\User;
use Illuminate\Http\Request;
use SimpleSoftwareIO\QrCode\Facades\QrCode;

use PhpOffice\PhpWord\IOFactory;
use PhpOffice\PhpWord\PhpWord;
use PhpOffice\PhpWord\Settings;

/**
 * Class RefillController
 * @package App\Http\Controllers
 */
class RefillController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        $query = Refill::query();

        $search = $request->search ?? '';

        if($search != '') {
            $query->where('name', 'like', '%' . $request->search . '%');
        }

        $refills = $query->paginate(30);

        return view('refill.index', compact('refills', 'search'))
            ->with('i', (request()->input('page', 1) - 1) * $refills->perPage());
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $refill = new Refill();

        $brands = Brand::query()->pluck('name', 'id');

        return view('refill.create', compact('refill', 'brands'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        request()->validate(Refill::$rules);

        $refill = Refill::create($request->all());

        $refill->quantity = $request->total;
        $refill->save();

        $uploadedFile = $request->file('invoice');

        //upload file
        if($uploadedFile != null) {
            $uploadedFile->storeAs('invoices', $refill->id.'.'.$uploadedFile->getClientOriginalExtension());

            $refill->invoice = 'invoices/'.$refill->id.'.'.$uploadedFile->getClientOriginalExtension();
        }

        $refill->save();

        return redirect()->route('refills.index', ['__cx' => time()])
            ->with('success', 'Repuesto creado con éxito.');
    }

    /**
     * Display the specified resource.
     *
     * @param  int $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $refill = Refill::find($id);

        return view('refill.show', compact('refill'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $refill = Refill::find($id);
        $brands = Brand::query()->pluck('name', 'id');

        return view('refill.edit', compact('refill', 'brands'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request $request
     * @param  Refill $refill
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Refill $refill)
    {
        request()->validate(Refill::$rules);

        $refill->update($request->all());

        return redirect()->route('refills.index', ['__cx' => time()])
            ->with('success', 'Repuesto actualizado con éxito');
    }

    /**
     * @param int $id
     * @return \Illuminate\Http\RedirectResponse
     * @throws \Exception
     */
    public function destroy($id)
    {
        $refill = Refill::find($id)->delete();

        return redirect()->route('refills.index', ['__cx' => time()])
            ->with('success', 'Repuesto eliminado con éxito');
    }

    public function search(Request $request)
    {
        $query = $request->input('query');
        $refills = Refill::query()
            ->where('quantity', '>', 0)
            ->where('name', 'like', '%'.$query.'%')
            ->limit(20)
            ->get();

        $detail = $request->input('detail');

        return view('refill._table', compact('refills', 'detail'));
    }

    public function assign(Request $request)
    {
        Usage::query()->where('order_detail_id', $request->detail)->delete();

        if(isset($request->date_recharge)){
            $usage = new Usage();

            $usage->order_detail_id = $request->detail;
            $usage->is_recharge = true;
            $usage->date_recharge = $request->date_recharge ?? null;
            $usage->date_recharge_expiration = $request->date_recharge_expiration ?? null;
            $usage->state = 'APPROVED';

            $usage->save();
        }

        if($request->refill)
        foreach ($request->refill as $k => $refill)
        {
            $usage = new Usage();

            $usage->order_detail_id = $request->detail;
            $usage->refill_id = $refill;
            $usage->quantity = $request->quantity[$k];
            //$usage->action_id = $request->action[$k];
            $usage->date_recharge_expiration = $request->date_recharge_expiration ?? null;

            $usage->save();

            $refill = Refill::find($refill);
            $refill->quantity = $refill->quantity - $request->quantity[$k];
            $refill->save();
        }

        if($request->external){

            $templatePath = storage_path('app'. DIRECTORY_SEPARATOR .'formato_proveedor2.docx');
            $template = new \PhpOffice\PhpWord\TemplateProcessor($templatePath);

            $template->setImageValue('imagen', [
                'path' => public_path('app'. DIRECTORY_SEPARATOR .'img'. DIRECTORY_SEPARATOR .'logo.png'),
                'width' => 100,
                'height' => 100,
            ]);

            $template->setValue('orden', $request->detail);
            $template->setValue('dia', date('d'));
            $template->setValue('mes', date('m'));
            $template->setValue('anio', date('Y'));
            $orderDetail = OrderDetail::query()->find($request->detail);
            foreach ($request->external as $k => $external)
            {
                $usage = new Usage();

                $usage->order_detail_id = $request->detail;
                $usage->is_recharge = false;
                $usage->action_id = $external;

                $usage->save();

                $action = Action::query()->find($external);
                $template->setValue('proveedor', $action->provider);
                $template->setValue('accion', $action->action);



                $template->setValue('desc', $orderDetail->product->name);
                $template->setValue('brand', $orderDetail->product->brand->name);
                $template->setValue('category', $orderDetail->product->category->name);
                $template->setValue('class', $orderDetail->product->classe->name);
                $template->setValue('ability', $orderDetail->product->capacity);
                $template->setValue('serial', $orderDetail->serial_number);
                $template->setValue('observation', $orderDetail->observation);
                $template->setValue('quantity', $orderDetail->quantity);

            }

            $wordFile = storage_path('app/output/formato_proveedor_' . $request->detail . '.docx');
            $template->saveAs($wordFile);

            Settings::setPdfRendererName(Settings::PDF_RENDERER_TCPDF);
            Settings::setPdfRendererPath(base_path('vendor/tecnickcom/tcpdf'));

            $phpWord = IOFactory::load($wordFile, 'Word2007');

            $pdfFile = storage_path('app/output/formato_proveedor_' . $request->detail . '.pdf');
            $phpWord->save($pdfFile, 'PDF');

            $users = User::role('Operaciones')->get();

            foreach ($users as $user)
            {
                dispatch(new SendDocumentExternalJob($user, "Documento Para Provedor # " . $request->detail, $orderDetail, $pdfFile));
            }
        }


        return redirect()->route('dashboard', ['__cx' => time()])
            ->with('success', 'Repuestos añadidos con éxito');
    }

    public function assignAll(Request $request)
    {
        $order = Order::query()->find($request->order);
        foreach ($order->orderDetails as $detail)
        {
            Usage::query()
                ->where('order_detail_id', $detail->id)
                ->where('is_recharge', 1)
                ->delete();


                $usage = new Usage();

                $usage->order_detail_id = $detail->id;
                $usage->is_recharge = true;
                $usage->date_recharge = $request->date_recharge ?? null;
                $usage->date_recharge_expiration = $request->date_recharge_expiration ?? null;
                $usage->state = 'APPROVED';

                $usage->save();
        }

        return redirect()->route('dashboard', ['__cx' => time()])
            ->with('success', 'Repuestos añadidos con éxito');
    }

    public function done(Request $request)
    {
        $detail = OrderDetail::query()->find($request->detail);

        $detail->state = 'DONE';
        $detail->save();

        $dataQr = [
            'order' => $detail->order_id,
            'detail' => $detail->id,
            'product' => $detail->product_id,
            'date' => date('Y-m-d H:i:s'),
            'coordinates' => $detail->coordinates ?? [],
            'customer' => [
                'id' => $detail->id,
                'name' => $detail->order->customer->name,
                'coordinates' => $detail->order->customer->coordinates,
            ],
        ];

        QrCode::size(400)
            ->format('png')
            ->generate(json_encode($dataQr), storage_path('app/qr/'.$detail->id.'.png'));

        /** @var Order $order */
        $order = $detail->order;

        $totalDetil = $order->orderDetails->count();
        $totalDetilDone = OrderDetail::query()->where('order_id', $order->id)->where('state', 'DONE')->count();

        if($totalDetil == $totalDetilDone){
            $order->state = 'DONE';
            $order->save();
        }

        return redirect()->route('dashboard', ['__cx' => time()])
            ->with('success', 'Detaller de orden completada con éxito');
    }

    public function doneOrder(Request $request, Order $order)
    {

        foreach ($order->orderDetails as $detail){
            /** @var OrderDetail $detail */
            if($detail->state == 'DONE'){
                continue;
            }

            $detail->state = 'DONE';
            $detail->save();

            $dataQr = [
                'order' => $detail->order_id,
                'detail' => $detail->id,
                'product' => $detail->product_id,
                'date' => date('Y-m-d H:i:s'),
                'coordinates' => $detail->coordinates ?? [],
                'customer' => [
                    'id' => $detail->id,
                    'name' => $detail->order->customer->name,
                    'coordinates' => $detail->order->customer->coordinates,
                ],
            ];

            QrCode::size(400)
                ->format('png')
                ->generate(json_encode($dataQr), storage_path('app/qr/'.$detail->id.'.png'));

            $totalDetil = $order->orderDetails->count();
            $totalDetilDone = OrderDetail::query()->where('order_id', $order->id)->where('state', 'DONE')->count();

            if($totalDetil == $totalDetilDone){
                $order->state = 'DONE';
                $order->save();
            }
        }

        return redirect()->route('dashboard', ['__cx' => time()])
            ->with('success', 'Orden completada con éxito');
    }

    public function download(Refill $refill)
    {
        //response file
        return response()->download(storage_path('app/'.$refill->invoice));
    }
}
